<?php

namespace Drupal\group_member_mgmt\GroupMembership;

use Drupal\Core\Access\AccessResult;
use Drupal\Core\Session\AccountInterface;
use Drupal\group\Entity\GroupContentInterface;
use Drupal\group\Entity\GroupInterface;
use Drupal\group\Plugin\GroupContentAccessControlHandler;

/**
 * Class GroupMembershipAccessControlHandler.
 */
class GroupMembershipAccessControlHandler extends GroupContentAccessControlHandler {

  /**
   * {@inheritdoc}
   */
  public function relationAccess(GroupContentInterface $group_content, $operation, AccountInterface $account, $return_as_object = FALSE) {
    if (!$group_content->hasField('group_roles')) {
      return parent::relationAccess($group_content, $operation, $account, $return_as_object);
    }

    // Look for access to memberships with no roles.
    if (count($group_content->group_roles->referencedEntities()) == 0) {
      $perm = $operation . ' group_membership content group-role:' . $group_content->getGroup()->getGroupType()->id() . '-member';
      if ($group_content->getGroup()->hasPermission($perm, $account)) {
        return AccessResult::allowed();
      }
    }

    // Loop through roles assigned to group content. Allow if user has access
    // via applicable permission.
    foreach ($group_content->group_roles->referencedEntities() as $group_role) {
      /** @var \Drupal\group\Entity\GroupRoleInterface $group_role */
      $perm = $operation . ' group_membership content group-role:' . $group_role->id();
      if ($group_content->getGroup()->hasPermission($perm, $account)) {
        return AccessResult::allowed();
      }
    }

    // Fall back to standard permissions and access.
    return parent::relationAccess($group_content, $operation, $account, $return_as_object);
  }

  /**
   * {@inheritdoc}
   */
  public function relationCreateAccess(GroupInterface $group, AccountInterface $account, $return_as_object = FALSE) {
    // Allow for the member (no role) permission.
    $createPerm = 'create group_membership content group-role:' . $group->getGroupType()->id() . '-member';
    if ($group->hasPermission($createPerm, $account)) {
      return AccessResult::allowed();
    }

    // Fall back to standard permissions and access.
    return parent::relationCreateAccess($group, $account, $return_as_object);
  }

}
